#include <Bounce2.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <LED.h>

#define LFLIP_PIN 3 //Left Flipper Button
#define COIN_PIN 4 //Start Button
#define RFLIP_PIN 5 //Right Flipper Button
#define FHDP_PIN 6 //500 Points Pin
#define THSP_PIN 7 //Ball Drain Switch
#define A_PIN 8 //LEDs
#define W_PIN 9
#define E_PIN 10
#define S_PIN 11
#define O_PIN 12
#define M_PIN 13
#define E2_PIN 14
#define LFLIPSOL_PIN 15 //Left Flipper Solenoid
#define RFLIPSOL_PIN 16 //Right Flipper Solenoid

LiquidCrystal_I2C lcdDisplay(0x27,16,2); //Defines LCD Display as lcdDisplay object

int gameState = 0; //Sets game state, 0 means attract, 1 means game, 2 means entering initials, 3 means credits
int credits = 0; //How much money is in the game
long int score = 0; //Sets game score to 0
long int highScore = 0;
int ball = 1; //Defaults the ball in play to 1
int multiplier = 1; //Default multiplyer is 1x
int tempScore = 0;
boolean gameOver = 0; // Is the game over?

Bounce coinButton = Bounce();
Bounce fivePointButton = Bounce();
Bounce thousandPointButton = Bounce();

LED aLed = LED(A_PIN);
LED wLed = LED(W_PIN);
LED eLed = LED(E_PIN);
LED sLed = LED(S_PIN);
LED oLed = LED(O_PIN);
LED mLed = LED(M_PIN);
LED e2Led = LED(E2_PIN);

long previousMillis = 0; 
long interval = 1000; 

void setup() {
  
//  pinMode(POWER_PIN, INPUT); // Power Button
  pinMode(LFLIP_PIN, INPUT); // Left Flipper
  pinMode(COIN_PIN, INPUT); // Coin Button
  pinMode(RFLIP_PIN, INPUT); // Right Flipper
  pinMode(FHDP_PIN, INPUT); // 500 Points
  pinMode(THSP_PIN, INPUT); // 1000 Points
  
  pinMode(LFLIPSOL_PIN, OUTPUT); // Left Flipper Solenoid
  pinMode(RFLIPSOL_PIN, OUTPUT); // Right Flipper Solenoid
  
  coinButton.attach(COIN_PIN);
  fivePointButton.attach(FHDP_PIN);
  thousandPointButton.attach(THSP_PIN);
  
  coinButton.interval(50);
  fivePointButton.interval(50);
  thousandPointButton.interval(50);
  
  lcdDisplay.init();
  lcdDisplay.backlight();
  lcdDisplay.print("AWESOME PINBALL!");
  lcdDisplay.setCursor(0, 1);
  lcdDisplay.print(" Build 11-11-14 ");
  delay(2000);
  lcdDisplay.clear();
  
}

void loop() {
  unsigned long currentMillis = millis();
  
  while(gameState == 0) { //We're in Attract Mode
    lcdDisplay.setCursor(0,0);
    
    lcdDisplay.print("  Press Start!  ");
    lcdDisplay.setCursor(0, 1);
    lcdDisplay.print("Hgh Scr:");
    lcdDisplay.print(highScore);
    
    if(digitalRead(COIN_PIN) == 1) { //Somebody's pressed the start button!
      gameState = 1;
      lcdDisplay.clear();
    }
    
    aLed.toggle();
    wLed.toggle();
    eLed.toggle();
    sLed.toggle();
    oLed.toggle();
    mLed.toggle();
    e2Led.toggle(); 
    delay(50);
  }
    while(gameState == 1) { //This is where the game actually runs
      if(ball == 4) { //The game is over!
        gameState = 2;
      }
      
      if(digitalRead(LFLIP_PIN) == 1) {
        digitalWrite(LFLIPSOL_PIN, HIGH);
      }
      else {
        digitalWrite(LFLIPSOL_PIN, LOW);
      }
      
      if(digitalRead(RFLIP_PIN) == 1) {
        digitalWrite(RFLIPSOL_PIN, HIGH);
      }
      else {
        digitalWrite(RFLIPSOL_PIN, LOW);
      }
      
      //Start by refreshing the display
      lcdDisplay.setCursor(0,0);
      lcdDisplay.print("Score:");
      lcdDisplay.print(score);
      
      lcdDisplay.setCursor(0,1);
      lcdDisplay.print("Ball:");
      lcdDisplay.print(ball);
      lcdDisplay.print(" Mult:");
      lcdDisplay.print(multiplier);
      lcdDisplay.print("x");
      
      //Next, let's see if the someone scored
      if(fivePointButton.update() == 1) {
        score += (250 * multiplier);
        tempScore += (250 * multiplier);
      }
      
      if(digitalRead(THSP_PIN) == 1) {
        //score += (500 * multiplier);
        //tempScore += (500 * multiplier);
        delay(500); 
        lcdDisplay.clear();
        lcdDisplay.setCursor(0,0);
        lcdDisplay.print("Bye Bye Ball! :(");
        ball++;
        delay(4000);
        lcdDisplay.clear(); 
      }
      
      //Set the LEDs accordingly
      if(tempScore == 0) {
        aLed.off();
        wLed.off();
        eLed.off();
        sLed.off();
        oLed.off();
        mLed.off();
        e2Led.off();
      }
      if(tempScore < 1000) {
        aLed.on();
      }
      if(tempScore > 1000 && tempScore < 2000) {
        aLed.on();
        wLed.on();
      }
      if(tempScore > 2000 && tempScore < 3000) {
        aLed.on();
        wLed.on();
        eLed.on();
      }
      if(tempScore > 3000 && tempScore < 4000) {
        aLed.on();
        wLed.on();
        eLed.on();
        sLed.on();
      }
      if(tempScore > 4000 && tempScore < 5000) {
        aLed.on();
        wLed.on();
        eLed.on();
        sLed.on();
        oLed.on();
      }
      if(tempScore > 5000 && tempScore < 6000) {
        aLed.on();
        wLed.on();
        eLed.on();
        sLed.on();
        oLed.on();
        mLed.on();
      }
      if(tempScore > 6000 && tempScore < 7000) {
        aLed.on();
        wLed.on();
        eLed.on();
        sLed.on();
        oLed.on();
        mLed.on();
        e2Led.on();
      }
      if(tempScore > 7000) {
        tempScore = 0;
        aLed.off();
        wLed.off();
        eLed.off();
        sLed.off();
        oLed.off();
        mLed.off();
        e2Led.off();
        if(multiplier < 4) {
        multiplier = 1;
        }
        else {
          multiplier = 1;
        }
      }
      while(gameState == 2) { //High Score Check
        if(score > highScore) {
          lcdDisplay.clear();
          lcdDisplay.setCursor(0,0);
          lcdDisplay.print(" New High Score ");
          
          lcdDisplay.setCursor(0,1);
          lcdDisplay.print("Score:");
          lcdDisplay.print(score);
          
          highScore = score;
          delay(5000);
          gameState = 0;
          lcdDisplay.clear();
          score = 0;
          ball = 1;
          
        }
        else {
          lcdDisplay.clear();
          lcdDisplay.setCursor(0,0);
          lcdDisplay.print("Awesome! See ya!");
          delay(5000);
          gameState = 0;
          lcdDisplay.clear();
          score = 0;
          ball = 1;
          
        }
      }
    }
  }
